from __future__ import (absolute_import, print_function, unicode_literals)
from acli.services.s3 import (s3_list)
from acli.config import Config
from moto import mock_s3

import pytest
from boto3.session import Session
session = Session(region_name="eu-west-1")


@pytest.yield_fixture(scope='function')
def s3_bucket():
    """S3 mock service"""
    mock = mock_s3()
    mock.start()
    s3_client = session.client('s3')
    s3_client.create_bucket(Bucket='test_bucket_1')
    yield s3_client.list_buckets()
    mock.stop()


config = Config(cli_args={'--region': 'eu-west-1',
                          '--access_key_id': 'AKIAIOSFODNN7EXAMPLE',
                          '--secret_access_key': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'})


def test_s3_list_service(s3_bucket):
    with pytest.raises(SystemExit):
        assert s3_list(aws_config=config)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import (output_ascii_table, output_ascii_table_list, dash_if_none)
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def output_route53_list(zones=None):
    """
    @type zones: list | dict
    """
    if isinstance(zones, dict):
        zones = [zones]
    for hosted_zone_dict in zones:
        td = list()
        table_header = [Color('{autoblue}id{/autoblue}'), Color('{autoblue}name{/autoblue}'),
                        Color('{autoblue}count{/autoblue}'), Color('{autoblue}comment{/autoblue}'),
                        Color('{autoblue}private zone{/autoblue}')]
        hosted_zones = sorted(hosted_zone_dict.get('HostedZones'), key=lambda k: k['Name'])
        for hosted_zone in hosted_zones:
            zone_id = dash_if_none(hosted_zone.get('Id'))
            zone_name = dash_if_none(hosted_zone.get('Name'))
            record_count = str(hosted_zone.get('ResourceRecordSetCount'))
            comment = dash_if_none(hosted_zone.get('Config').get('Comment'))
            private_zone = dash_if_none(hosted_zone.get('Config').get('PrivateZone'))
            td.append([zone_id,
                       zone_name,
                       record_count,
                       comment,
                       private_zone])
    output_ascii_table_list(table_title=Color('{autowhite}route53 zones{/autowhite}'),
                            table_data=td,
                            table_header=table_header,
                            inner_heading_row_border=True)
    exit(0)


def get_record_set_values(resource_records):
    """
    @type resource_records: list
    """
    out = list()
    for record in resource_records:
        out.append(record.get('Value'))
    return out


def output_route53_info(zone=None, record_sets=None):
    """
    @type zone: zone
    @type record_sets: ResourceRecordSets
    """
    td = list()
    td.append([Color('{autoblue}id{/autoblue}'),
               dash_if_none(zone.get('HostedZone').get('Id', None))])
    td.append([Color('{autoblue}name{/autoblue}'),
               dash_if_none(zone.get('HostedZone').get('Name', None))])
    td.append([Color('{autoblue}records{/autoblue}'),
               dash_if_none(zone.get('HostedZone').get('ResourceRecordSetCount', None))])
    td.append([Color('{autoblue}comment{/autoblue}'),
               dash_if_none(zone.get('HostedZone').get('Config').get('Comment', None))])
    td.append([Color('{autoblue}private{/autoblue}'),
               dash_if_none(zone.get('HostedZone').get('Config').get('PrivateZone', None))])
    td.append([Color('{autoblue}name servers{/autoblue}'),
               "\n".join(zone['DelegationSet']['NameServers'])])
    td.append([Color('{autoblue}records{/autoblue}'), ' '])
    td.append(['{0}'.format("-" * 12), '{0}'.format("-" * 20)])
    for record_set in record_sets['ResourceRecordSets']:
        td.append([Color('{autoblue}name{/autoblue}'),
                   record_set['Name']])
        td.append([Color('{autoblue} type{/autoblue}'),
                   record_set['Type']])
        td.append([Color('{autoblue} ttl{/autoblue}'),
                   str(record_set['TTL'])])
        td.append([Color('{autoblue} values{/autoblue}'),
                   "\n".join(get_record_set_values(record_set['ResourceRecords']))])
    output_ascii_table(table_title=Color('{autowhite}zone info{/autowhite}'),
                       table_data=td)
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.asg import (output_asg_list, output_asg_info,
                             output_lc_list, output_lc_info)
from acli.connections import get_client


def asg_list(aws_config=None):
    """
    @type aws_config: Config
    """
    asg_client = get_client(client_type='autoscaling', config=aws_config)
    all_asgs = asg_client.describe_auto_scaling_groups().get('AutoScalingGroups', None)
    if all_asgs:
        output_asg_list(asg_list=all_asgs)
    else:
        exit("No auto scaling groups were found.")


def asg_info(aws_config=None, asg_name=None):
    """
    @type aws_config: Config
    @type asg_name: unicode
    """
    asg_client = get_client(client_type='autoscaling', config=aws_config)
    asg = asg_client.describe_auto_scaling_groups(AutoScalingGroupNames=[asg_name])
    if asg.get('AutoScalingGroups'):
        output_asg_info(asg=asg.get('AutoScalingGroups')[0])
    else:
        exit("Auto Scaling Group: {0} not found.".format(asg_name))


def asg_delete(aws_config=None, asg_name=None):
    """
    @type aws_config: Config
    @type asg_name: unicode
    """
    asg_client = get_client(client_type='autoscaling', config=aws_config)
    asg = asg_client.describe_auto_scaling_groups(AutoScalingGroupNames=[asg_name])
    if asg.get('AutoScalingGroups'):
        if len(asg.get('AutoScalingGroups')):
            asg_instance = asg.get('AutoScalingGroups')[0]
            asg_instance_name = asg_instance.get('AutoScalingGroupName')
            asg_client.update_auto_scaling_group(
                                                AutoScalingGroupName=asg_instance_name,
                                                MinSize=0,
                                                MaxSize=0,
                                                )
            asg_client.delete_auto_scaling_group(
                                                AutoScalingGroupName=asg_instance_name,
                                                ForceDelete=False
                                                )
        exit("Auto Scaling Group {0} is being deleted.".format(asg_instance_name))
    else:
        exit("Auto Scaling Group: {0} not found.".format(asg_name))


def lc_list(aws_config=None):
    """
    @type aws_config: Config
    """
    asg_client = get_client(client_type='autoscaling', config=aws_config)
    all_lcs = asg_client.describe_launch_configurations().get('LaunchConfigurations', None)
    if all_lcs:
        output_lc_list(lc_list=all_lcs)
    else:
        exit("No launch configurations were found.")


def lc_info(aws_config=None, lc_name=None):
    """
    @type aws_config: Config
    @type lc_name: unicode
    """
    asg_client = get_client(client_type='autoscaling', config=aws_config)
    lc = asg_client.describe_launch_configurations(LaunchConfigurationNames=[lc_name])
    lc_details = lc.get('LaunchConfigurations', None)
    if lc_details:
        output_lc_info(lc=lc_details[0])
    else:
        exit("Launch Configuration: {0} not found.".format(lc_name))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import (output_ascii_table, output_ascii_table_list, dash_if_none)
import pprint
import json
import six
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def default(obj):
    """Default JSON serializer."""
    import calendar, datetime

    if isinstance(obj, datetime.datetime):
        if obj.utcoffset() is not None:
            obj = obj - obj.utcoffset()
        millis = int(
            calendar.timegm(obj.timetuple()) * 1000 +
            obj.microsecond / 1000
        )
        return millis
    raise TypeError('Not sure how to serialize %s' % obj)


def get_ec2_instance_tags(ec2_instance=None, tag_key=None,
                          max_length=40):
    """
    @type ec2_instance: ec2.Instance
    @type tag_key: unicode
    @type max_length: int
    """
    if ec2_instance.get('Tags', None):
        ret = []
        for tag in ec2_instance.get('Tags'):
            if tag_key and tag.get('Key') == tag_key:
                return tag.get('Value', "-")
            else:
                val = tag.get('Value', None)
                if val and len(val) > max_length:
                    val = "{0}...".format(val[:max_length-3])
                ret.append('{0}:{1}\n'.format(tag.get('Key'), val))
        return "".join(ret).rstrip()
    else:
        return ""


def get_interfaces(interfaces):
    """
    @type interfaces: list
    """
    ret = list()
    for interface in interfaces:
        ret.append("{0}\n".format(interface.get('NetworkInterfaceId')))
        ret.append(" Attachment:\n")
        for akey, avalue in six.iteritems(interface.get('Attachment')):
                ret.append("  {}:{}\n".format(str(akey), str(avalue)))
        ret.append(" Private IP Addresses:\n")
        for private_ip_address in interface.get('PrivateIpAddresses'):
                for pkey, pvalue in six.iteritems(private_ip_address):
                    if pkey == "Association":
                        ret.append("  Association:\n")
                        for qqkey, qqvalue in six.iteritems(pvalue):
                            ret.append("   {}:{}\n".format(qqkey, qqvalue))
                    else:
                        ret.append("  {}:{}\n".format(str(pkey), str(pvalue)))
        ret.append(" Security Groups:\n")
        for group in interface.get('Groups'):
            for gkey, gvalue in six.iteritems(group):
                ret.append("  {}:{}\n".format(str(gkey), str(gvalue)))
        for key, value in six.iteritems(interface):
            if str(key) not in ("Attachment", "NetworkInterfaceId",
                                "PrivateIpAddresses", "Groups", "Association",
                                "PrivateIpAddress"):
                ret.append(" {}:{}\n".format(str(key), str(value)))
    return ("".join(ret)).rstrip()


def get_placement_details(placement):
    """
    @type placement: dict
    """
    ret = []
    for key, value in six.iteritems(placement):
        ret.append("{0}:{1}".format(key, value))
    if ret:
        return "\n".join(ret)


def colour_state(state=None):
    if not state:
        return Color('{autoblack}-{/autoblack}')
    elif state == 'running':
        return Color('{autogreen}'+state+'{/autogreen}')
    elif state in ('stopped', 'stopping', 'shutting-down', 'terminated'):
        return Color('{autored}'+state+'{/autored}')
    elif state in ('rebooting', 'pending'):
        return Color('{autoyellow}'+state+'{/autoyellow}')


def output_ec2_list(instances=None):
    """
    @type instances: list
    """
    td = list()
    table_header = [Color('{autoblue}id{/autoblue}'), Color('{autoblue}name{/autoblue}'),
                    Color('{autoblue}state{/autoblue}'), Color('{autoblue}type{/autoblue}'),
                    Color('{autoblue}image{/autoblue}'),
                    Color('{autoblue}public ip{/autoblue}'), Color('{autoblue}private ip{/autoblue}')]
    instances = sorted(instances,
                       key=lambda k: get_ec2_instance_tags(ec2_instance=k, tag_key='Name'))
    for instance in instances:
        if instance:
            instance_id = instance.get('InstanceId')
            instance_state = colour_state(instance.get('State').get('Name', None))
            instance_type = dash_if_none(str(instance.get('InstanceType', None)))
            image_id = dash_if_none(instance.get('ImageId'))
            public_ip = dash_if_none(instance.get('PublicIpAddress', None))
            private_ip = dash_if_none(instance.get('PrivateIpAddress', None))
            instance_name = dash_if_none(get_ec2_instance_tags(ec2_instance=instance, tag_key='Name'))
            td.append([instance_id,
                       instance_name,
                       instance_state,
                       instance_type,
                       image_id,
                       public_ip,
                       private_ip])
    output_ascii_table_list(table_title=Color('{autowhite}ec2 instances{/autowhite}'),
                            table_header=table_header,
                            table_data=td,
                            inner_heading_row_border=True)
    exit(0)


def short_instance_profile(instance_profile=None):
    """
    @type instance_profile: dict
    """
    if instance_profile and instance_profile.get('Arn', None):
        return instance_profile.get('Arn', None)


def get_sec_group_names(groups=None):
    """
    @type groups: list
    """
    sec_group_names = [x.name for x in groups]
    return ",".join(sec_group_names)


def get_sec_groups_name_and_id(groups=None):
    """
    @type groups: list
    """
    ret = []
    for group in groups:
        ret.append('GroupName:{}\nGroupId:{}'.format(group.get('GroupName', '-'), group.get('GroupId', '-')))
    return "\n".join(ret).rstrip()


def get_interface_ids(interfaces=None):
    """
    @type interfaces: list
    """
    interface_ids = [x.id for x in interfaces]
    return ",".join(interface_ids)


def get_state_reason(state_reason=None):
    """
    @type state_reason: dict
    """
    if state_reason:
        return state_reason.get('Message', None)


def get_block_devices(bdms=None):
    """
    @type bdms: list
    """
    ret = ""
    if bdms:
        for bdm in bdms:
            ret += "{0}\n".format(bdm.get('DeviceName', '-'))
            ebs = bdm.get('Ebs', None)
            if ebs:
                ret += " Status: {0}\n".format(ebs.get('Status', '-'))
                ret += " Snapshot Id: {0}\n".format(ebs.get('SnapshotId', '-'))
                ret += " Volume Size: {0}\n".format(ebs.get('VolumeSize', '-'))
                ret += " Volume Type: {0}\n".format(ebs.get('VolumeType', '-'))
                ret += " Encrypted: {0}\n".format(str(ebs.get('Encrypted', '-')))
                ret += " Delete on Termination: {0}\n".format(ebs.get('DeleteOnTermination', '-'))
                ret += " Attach Time: {0}\n".format(str(ebs.get('AttachTime', '-')))
        return ret.rstrip()
    else:
        return ret


def output_ec2_info(instance=None):
    """
    @type instance: ec2.Instance
    """
    td = list()
    td.append([Color('{autoblue}id{/autoblue}'),
               instance.get('InstanceId')])
    td.append([Color('{autoblue}name{/autoblue}'),
               dash_if_none(get_ec2_instance_tags(ec2_instance=instance, tag_key='Name'))])
    td.append([Color('{autoblue}groups{/autoblue}'),
               dash_if_none(get_sec_groups_name_and_id(instance.get('SecurityGroups')))])
    td.append([Color('{autoblue}public ip{/autoblue}'),
               dash_if_none(instance.get('PublicIpAddress', None))])
    td.append([Color('{autoblue}public dns name{/autoblue}'),
               dash_if_none(instance.get('PublicDnsName', None))])
    td.append([Color('{autoblue}private ip{/autoblue}'),
               dash_if_none(instance.get('PrivateIpAddress', None))])
    td.append([Color('{autoblue}private dns name{/autoblue}'),
               dash_if_none(instance.get('PrivateDnsName', None))])
    td.append([Color('{autoblue}state{/autoblue}'),
               colour_state(instance.get('State')['Name'])])
    td.append([Color('{autoblue}key name{/autoblue}'),
               dash_if_none(instance.get('KeyName', None))])
    td.append([Color('{autoblue}instance type{/autoblue}'),
               dash_if_none(instance.get('InstanceType', None))])
    td.append([Color('{autoblue}launch time{/autoblue}'),
               str(instance.get('LaunchTime'))])
    td.append([Color('{autoblue}image id{/autoblue}'),
               dash_if_none(instance.get('ImageId'))])
    td.append([Color('{autoblue}placement{/autoblue}'),
               get_placement_details(instance.get('Placement'))])
    td.append([Color('{autoblue}monitored{/autoblue}'),
               'enabled' if instance.get('Monitoring')['State'] == 'enabled' else 'disabled'])
    td.append([Color('{autoblue}subnet id{/autoblue}'),
               dash_if_none(instance.get('SubnetId'))])
    td.append([Color('{autoblue}vpc id{/autoblue}'),
               dash_if_none(instance.get('VpcId'))])
    td.append([Color('{autoblue}root device type{/autoblue}'),
               dash_if_none(instance.get('RootDeviceType', None))])
    td.append([Color('{autoblue}state transition reason{/autoblue}'),
               dash_if_none(instance.get('StateTransitionReason', None))])
    td.append([Color('{autoblue}ebs optimized{/autoblue}'),
               dash_if_none(instance.get('EbsOptimized', None))])
    td.append([Color('{autoblue}instance profile{/autoblue}'),
               dash_if_none(short_instance_profile(instance.get('IamInstanceProfile', None)))])
    td.append([Color('{autoblue}tags{/autoblue}'),
               get_ec2_instance_tags(ec2_instance=instance)])
    td.append([Color('{autoblue}block devices{/autoblue}'),
               get_block_devices(instance.get('BlockDeviceMappings'))])
    td.append([Color('{autoblue}interfaces{/autoblue}'),
               dash_if_none(get_interfaces(instance.get('NetworkInterfaces', None)))])
    output_ascii_table(table_title=Color('{autowhite}instance info{/autowhite}'),
                       table_data=td)
    exit(0)


def trim_creation_date(creation_date=None):
    if creation_date:
        return creation_date[:-5].replace('T', ' ')


def output_ami_list(output_media='console', amis=None):
    """
    @type output_media: unicode
    @type amis: list
    """
    amis = sorted(amis, key=lambda k: k.get('CreationDate'), reverse=True)
    if output_media == 'console':
        td = [[Color('{autoblue}image id{/autoblue}'),
               Color('{autoblue}name{/autoblue}'),
               Color('{autoblue}created (UTC){/autoblue}')]]
        for ami in amis:
            td.append([ami.get('ImageId'),
                       dash_if_none(ami.get('Name')),
                       dash_if_none(trim_creation_date(ami.get('CreationDate')))])
        output_ascii_table(table_title=Color('{autowhite}AMIs{/autowhite}'),
                           inner_heading_row_border=True,
                           table_data=td)
    exit(0)


def output_ami_permissions(perms=None):
    """
    @type perms: dict
    """
    out = ""
    for i, (key, value) in enumerate(six.iteritems(perms)):
        out += "{0},{1}".format(key, str(value))
        if i < len(perms)-1:
            out += "\n"
    return out


def get_product_codes(product_codes=None):
    """
    @type product_codes: dict
    """
    if product_codes:
        out = ""
        for prodcode in product_codes:
            out += "{0}/{1}".format(prodcode.get('ProductCodeId'), prodcode.get('ProductCodeType'))
        return out


def output_ami_info(output_media='console', ami=None):
    """
    @type output_media: unicode
    @type ami: ec2.Ami
    """
    if output_media == 'console':
        td = list()
        td.append([Color('{autoblue}id{/autoblue}'),
                   ami.get('ImageId')])
        td.append([Color('{autoblue}name{/autoblue}'),
                   ami.get('Name')])
        td.append([Color('{autoblue}created (UTC){/autoblue}'),
                   dash_if_none(trim_creation_date(ami.get('CreationDate')))])
        td.append([Color('{autoblue}description{/autoblue}'),
                   dash_if_none(ami.get('Description'))])
        td.append([Color('{autoblue}hypervisor{/autoblue}'),
                   dash_if_none(ami.get('Hypervisor'))])
        td.append([Color('{autoblue}public{/autoblue}'),
                   dash_if_none(str(ami.get('Public')))])
        td.append([Color('{autoblue}kernel id{/autoblue}'),
                   dash_if_none(ami.get('KernelId'))])
        td.append([Color('{autoblue}location{/autoblue}'),
                   dash_if_none(ami.get('ImageLocation'))])
        td.append([Color('{autoblue}owner id{/autoblue}'),
                   dash_if_none(ami.get('OwnerId'))])
        td.append([Color('{autoblue}owner alias{/autoblue}'),
                   dash_if_none(ami.get('ImageOwnerAlias'))])
        td.append([Color('{autoblue}platform{/autoblue}'),
                   dash_if_none(ami.get('Platform'))])
        td.append([Color('{autoblue}product codes{/autoblue}'),
                   dash_if_none(get_product_codes(ami.get('ProductCodes')))])
        td.append([Color('{autoblue}root device name{/autoblue}'),
                   dash_if_none(ami.get('RootDeviceName'))])
        td.append([Color('{autoblue}root device typr{/autoblue}'),
                   dash_if_none(ami.get('RootDeviceType'))])
        td.append([Color('{autoblue}sriov net support{/autoblue}'),
                   dash_if_none(ami.get('SriovNetSupport'))])
        td.append([Color('{autoblue}state{/autoblue}'),
                   dash_if_none(ami.get('State'))])
        td.append([Color('{autoblue}virtualisation type{/autoblue}'),
                   dash_if_none(ami.get('VirtualizationType'))])
        td.append([Color('{autoblue}block device mapping{/autoblue}'),
                   get_block_devices(bdms=ami.get('BlockDeviceMappings'))])
        output_ascii_table(table_title=Color('{autowhite}AMI info{/autowhite}'),
                           table_data=td)
    exit(0)


def output_ec2_summary(output_media='console', summary=None):
    """
    @type output_media: unicode
    @type summary: dict
    """
    if output_media == 'console':
        td = list()
        td.append([Color('{autoblue}running instances{/autoblue}'), str(summary.get('instances', '0')),
                   Color('{autoblue}load balancers{/autoblue}'), str(summary.get('elbs', '0'))])
        td.append([Color('{autoblue}elastic IPs{/autoblue}'), str(summary.get('eips', '0')),
                   Color('{autoblue}AMIs{/autoblue}'), str(summary.get('amis', '0'))])
        td.append([Color('{autoblue}security groups{/autoblue}'), str(summary.get('secgroups', '0')),
                   '', ''])
        output_ascii_table(table_title=Color('{autowhite}ec2 summary{/autowhite}'),
                           table_data=td)
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.account import output_account_info
from acli.connections import get_client


def account_info(aws_config):
    """
    @type aws_config: Config
    """
    iam_client = get_client(client_type='iam', config=aws_config)
    users = iam_client.list_users(MaxItems=1)
    if users.get('Users', None):
        account_id = users.get('Users')[0]['Arn'].split(':')[4]
        aliases = iam_client.list_account_aliases().get('AccountAliases')
        output_account_info(account_id=account_id,
                            account_aliases=aliases)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.s3 import (output_s3_list, output_s3_info)
import botocore.exceptions
from acli.connections import get_client


def check_bucket_accessible(s3_client=None, bucket_name=None):
    try:
        s3_client.head_bucket(Bucket=bucket_name)
        return True
    except botocore.exceptions.ClientError:
        exit("Unable to access bucket.")
    except Exception as unhandled:
        exit('Unhandled exception: {0}'.format(unhandled))


def s3_list(aws_config=None, item=None):
    """
    @type aws_config: Config
    """
    s3_client = get_client(client_type='s3', config=aws_config)
    buckets = s3_client.list_buckets()
    if not item:
        if buckets.get('Buckets', None):
            output_s3_list(buckets=buckets.get('Buckets'))
        else:
            exit("No buckets found.")
    else:
        prefix = ''
        if item and '/' in item:
            path_elements = item.split('/')
            bucket_name = path_elements[0]
            prefix = "/".join(path_elements[1:])
        else:
            bucket_name = item
        check_bucket_accessible(s3_client=s3_client, bucket_name=bucket_name)
        try:
            objects = s3_client.list_objects(Bucket=bucket_name, Prefix=prefix, Delimiter='/')
            if not any((objects.get('CommonPrefixes', None),
                        (objects.get('Contents', None) and len(objects.get('Contents')) > 1))):
                exit('Nothing found in: {0}'.format(item[:-1]))
            common_prefixes = objects.get('CommonPrefixes', list())
            folders = list()
            for first_bit in common_prefixes:
                folders.append(first_bit)
            output_s3_list(objects=objects, folders=folders, item=item, bucket_name=bucket_name)
        except botocore.exceptions.ClientError as error:
            if 'NoSuchBucket' in error.response['Error']['Code']:
                exit('Bucket not found.')
            else:
                exit('Unhandled error: {0}'.format(error.response['Error']['Code']))


def s3_info(aws_config=None, item=None):
    """
    @type aws_config: Config
    @type item: unicode
    """
    s3_client = get_client(client_type='s3', config=aws_config)
    prefix = ''
    if item and '/' in item:
        path_elements = item.split('/')
        bucket_name = path_elements[0]
        prefix = "/".join(path_elements[1:])
        if prefix.endswith('/'):
            prefix = prefix[:-1]
    else:
        bucket_name = item
    check_bucket_accessible(s3_client=s3_client, bucket_name=bucket_name)
    try:
        s3_object = s3_client.list_objects(Bucket=bucket_name, Prefix=prefix, Delimiter='/')
        if 'Contents' in s3_object.keys():
            output_s3_info(s3_object=s3_object)
        else:
            # TODO: Decide on output or message
            print("path, not object was requested")
    except botocore.exceptions.ClientError as error:
        if 'NoSuchBucket' in error.response['Error']['Code']:
            exit('Bucket not found.')
        else:
            exit('Unhandled error: {0}'.format(error.response['Error']['Code']))



from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.route53 import (output_route53_list, output_route53_info)
from acli.services.route53 import (route53_list, route53_info)
from acli.config import Config
from moto import mock_route53

import pytest
from boto3.session import Session
session = Session(region_name="eu-west-1")


@pytest.yield_fixture(scope='function')
def route53_zone():
    """ELB mock service"""
    mock = mock_route53()
    mock.start()
    client = session.client('route53')
    client.create_hosted_zone(Name="testdns.aws.com", CallerReference='auniqueref',
                              HostedZoneConfig={'Comment': 'string', 'PrivateZone': False})
    yield client.list_hosted_zones()
    mock.stop()


config = Config(cli_args={'--region': 'eu-west-1',
                          '--access_key_id': 'AKIAIOSFODNN7EXAMPLE',
                          '--secret_access_key': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'})


def test_elb_list_service(route53_zone):
    with pytest.raises(SystemExit):
        assert route53_list(aws_config=config)


def test_elb_info_service(route53_zone):
    with pytest.raises(SystemExit):
        print(route53_zone)
        assert route53_info(aws_config=config, zone_id=route53_zone.get('HostedZones')[0].get('Id'))



# -*- coding: utf-8 -*-
"""Usage:
    acli asg (list | info <asg_name>) [options]
    acli asg (cpu | mem | net | delete) <asg_name> [options]
    -s, --start=<start_date>        metrics start-date
    -e, --end=<end_date>            metrics end-date
    -p, --period<period>            metrics period
    -i, --intervals=<intervals>     metrics intervals
    -o, --output=<output_type>      table, json, yaml or graph [default: graph].
    -h, --help
"""
from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt
if __name__ == '__main__':
    print(docopt(__doc__))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)


def output_ec2_cpu(dates=None, values=None,
                   instance_id=None, output_type=None):
    """
    @type dates: list
    @type values: list
    @type instance_id: unicode
    @type output_type: unicode
    """
    try:
        import matplotlib.pyplot as plt
        import matplotlib.dates as mdates
    except ImportError:
        exit('matplotlib required to output graphs.')
    if output_type in ('graph', None):
        plt.subplots_adjust(bottom=0.2)
        plt.xticks(rotation=25)
        ax = plt.gca()
        xfmt = mdates.DateFormatter('%Y-%m-%d %H:%M:%S')
        ax.xaxis.set_major_formatter(xfmt)
        plt.plot(dates, values)
        plt.gcf().autofmt_xdate()
        plt.title('CPU statistics for: {0}'.format(instance_id))
        plt.xlabel('Time (UTC)')
        plt.ylabel('CPU %')
        plt.grid(True)
        plt.ylim([0, 100])
        plt.show()
    exit(0)


def output_ec2_mem(dates=None, values=None, instance_id=None, output_type=None):
    """
    @type dates: list
    @type values: list
    @type instance_id: unicode
    @type output_type: unicode
    """
    try:
        import matplotlib.pyplot as plt
        import matplotlib.dates as mdates
    except ImportError:
        exit('matplotlib required to output graphs.')
    if output_type in ('graph', None):
        plt.subplots_adjust(bottom=0.2)
        plt.xticks(rotation=25)
        ax = plt.gca()
        xfmt = mdates.DateFormatter('%Y-%m-%d %H:%M:%S')
        ax.xaxis.set_major_formatter(xfmt)
        plt.plot(dates, values)
        plt.gcf().autofmt_xdate()
        plt.title(' Memory usage for: {0}'.format(instance_id))
        plt.xlabel('Time (UTC)')
        plt.ylabel('Memory Usage %')
        plt.grid(True)
        plt.show()
    exit(0)


def output_ec2_net(in_dates=None, in_values=None, out_dates=None,
                   out_values=None, instance_id=None, output_type=None):
    """
    @type in_dates: list
    @type in_values: list
    @type out_dates: list
    @type out_values: list
    @type instance_id: unicode
    @type output_type: unicode
    """
    try:
        import matplotlib.pyplot as plt
        import matplotlib.dates as mdates
    except ImportError:
        exit('matplotlib required to output graphs.')
    if output_type in ('graph', None):
        plt.subplots_adjust(bottom=0.2)
        plt.xticks(rotation=25)
        ax = plt.gca()
        xfmt = mdates.DateFormatter('%Y-%m-%d %H:%M:%S')
        ax.xaxis.set_major_formatter(xfmt)
        in_line = plt.plot(in_dates, in_values)
        out_line = plt.plot(out_dates, out_values)
        plt.setp(in_line, color='g', linewidth=2.0, label='inbound')
        plt.setp(out_line, color='b', linewidth=2.0, label='outbound')
        plt.gcf().autofmt_xdate()
        plt.title('Network statistics for: {0}'.format(instance_id))
        plt.xlabel('Time (UTC)')
        plt.ylabel('Network (Bytes/s)')
        handles, labels = ax.get_legend_handles_labels()
        ax.legend(handles, labels)
        plt.subplots_adjust(bottom=0.2)
        plt.xticks(rotation=25)
        plt.grid()
        plt.show()
        exit(0)


def output_ec2_vols(vols_datapoints=None, instance_id=None, output_type=None):
    """
    @type vols_datapoints: list
    @type instance_id: unicode
    @type output_type: unicode
    """
    try:
        import matplotlib.pyplot as plt
        import matplotlib.dates as mdates
    except ImportError:
        exit('matplotlib required to output graphs.')
    try:
        import numpy as np
    except ImportError:
        exit('install numpy.')
    if output_type in ('graph', None):
        num_plots = len(vols_datapoints)
        f, axarr = plt.subplots(num_plots, sharex=True, sharey=True)
        f.suptitle('Volumes for instance: {0}'.format(instance_id), fontsize=16)
        plt.ylabel('Bytes')
        if isinstance(axarr, np.ndarray):
            for index, vol_set in enumerate(vols_datapoints):
                read_dates = vol_set.get('read_dates')
                read_values = vol_set.get('read_values')
                write_dates = vol_set.get('write_dates')
                write_values = vol_set.get('write_values')
                axarr[index].set_title(vol_set.get('device_name'))
                axarr[index].grid(True)
                axarr[index].plot(write_dates, write_values, label='write')
                axarr[index].plot(read_dates, read_values, label='read')
                axarr[index].legend(loc="upper right",
                                    title=None,
                                    fancybox=False)
            plt.subplots_adjust(bottom=0.2)
            plt.xticks(rotation=25)
            plt.xlabel('Time (UTC)')
        else:
            read_dates = vols_datapoints[0].get('read_dates')
            read_values = vols_datapoints[0].get('read_values')
            write_dates = vols_datapoints[0].get('write_dates')
            write_values = vols_datapoints[0].get('write_values')
            axarr.set_title(vols_datapoints[0].get('device_name'))
            axarr.plot(write_dates, write_values, label='write')
            axarr.plot(read_dates, read_values, label='read')
            axarr.legend(loc="upper right",
                         title=None,
                         fancybox=False)
            axarr.grid(True)
            plt.subplots_adjust(bottom=0.2)
            plt.xticks(rotation=25)
            plt.xlabel('Time (UTC)')
        ax = plt.gca()
        ax.set_ylim(bottom=0)
        handles, labels = ax.get_legend_handles_labels()
        ax.legend(handles, labels)
        xfmt = mdates.DateFormatter('%Y-%m-%d %H:%M:%S')
        ax.xaxis.set_major_formatter(xfmt)
        plt.grid(True)
        plt.show()
        exit(0)


def output_asg_cpu(dates=None, values=None,
                   asg_name=None, output_type=None):
    """
    @type dates: list
    @type values: list
    @type asg_name: unicode
    @type output_type: unicode
    """
    try:
        import matplotlib.pyplot as plt
        import matplotlib.dates as mdates
    except ImportError:
        exit('matplotlib required to output graphs.')
    if output_type in ('graph', None):
        plt.subplots_adjust(bottom=0.2)
        plt.xticks(rotation=25)
        ax = plt.gca()
        xfmt = mdates.DateFormatter('%Y-%m-%d %H:%M:%S')
        ax.xaxis.set_major_formatter(xfmt)
        plt.plot(dates, values)
        plt.gcf().autofmt_xdate()
        plt.title('CPU statistics for: {0}'.format(asg_name))
        plt.xlabel('Time (UTC)')
        plt.ylabel('CPU %')
        plt.grid()
        plt.ylim([0, 100])
        plt.show()
        exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.ec2 import (output_ec2_list, output_ec2_info,
                             output_ami_list, output_ami_info,
                             output_ec2_summary)
from acli.connections import get_client
from botocore.exceptions import ClientError


def ec2_summary(aws_config=None):
    """
    @type aws_config: Config
    """
    elb_client = get_client(client_type='elb', config=aws_config)
    ec2_client = get_client(client_type='ec2', config=aws_config)
    elbs = len(elb_client.describe_load_balancers().get('LoadBalancerDescriptions'))
    instances = len(list(ec2_client.describe_instances().get('Reservations', [])))
    amis = len(list(ec2_client.describe_images(Owners=['self']).get('Images')))
    secgroups = len(ec2_client.describe_security_groups().get('SecurityGroups', 0))
    addresses = ec2_client.describe_addresses()['Addresses']
    eips = len([x for x, _ in enumerate(addresses)])
    summary = {'instances': instances, 'elbs': elbs, 'eips': eips,
               'amis': amis, 'secgroups': secgroups}
    output_ec2_summary(summary=summary)
    exit(0)


def ec2_list(aws_config=None):
    """
    @type aws_config: Config
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    instances_req = ec2_client.describe_instances()
    reservations = instances_req.get('Reservations')
    all_instances = list()
    for reservation in reservations:
        for instance in reservation.get('Instances'):
            all_instances.append(instance)

    if len(list(all_instances)):
        output_ec2_list(instances=all_instances)
    exit('No ec2 instances found.')


def ec2_info(aws_config=None, instance_id=None):
    """
    @type aws_config: Config
    @type instance_id: unicode
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    ec2_query = ec2_client.describe_instances(Filters=[{'Name': 'instance-id', 'Values': [instance_id]}])
    reservations = ec2_query.get('Reservations')
    try:
        instance = reservations[0].get('Instances')[0]
        if instance.get('InstanceId', None):
            output_ec2_info(instance=instance)
    except IndexError:
        raise SystemExit("Cannot find instance: {0}".format(instance_id))


def ec2_manage(aws_config=None, instance_id=None, action=None):
    """
    @type aws_config: Config
    @type instance_id: unicode
    @type action: unicode
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    reservations = ec2_client.describe_instances(InstanceIds=[instance_id])
    instance = reservations.get('Reservations')[0].get('Instances')[0]
    try:
        instance_id = instance.get('InstanceId')
        instance_state = instance['State']['Name']
        if instance_id:
            if action == 'stop':
                if instance_state in ('pending', 'rebooting', 'stopping', 'terminated', 'shutting-down'):
                    exit("Cannot stop instance as state is {0}.".format(instance_state))
                elif instance_state in ('stopping', 'stopped', 'shutting-down'):
                    exit("Instance {0} is already {1}.".format(instance_id, instance_state))
                else:
                    ec2_client.stop_instances(InstanceIds=[instance_id])
                    exit("Instance {0} stopping.".format(instance_id))
            if action == 'start':
                if instance_state in ('rebooting', 'stopping',
                                      'terminated', 'shutting-down'):
                    exit("Cannot start instance {0} as state is {1}.".format(instance_id,
                                                                             instance_state))
                elif instance_state in ('pending', 'rebooting',
                                        'stopping', 'terminated',
                                        'shutting-down', 'running'):
                    exit("Instance {0} is already {1}.".format(instance_id, instance_state))
                else:
                    ec2_client.start_instances(InstanceIds=[instance_id])
                    exit("Instance {0} starting.".format(instance_id))
            if action == 'reboot':
                if instance_state in ('pending', 'stopping', 'terminated', 'shutting-down'):
                    exit("Cannot reboot instance {0} as state is {1}.".format(instance_id,
                                                                              instance_state))
                elif instance_state == 'rebooting':
                    exit("Instance {0} is already {1}.".format(instance_id, instance_state))
                else:
                    ec2_client.reboot_instances(InstanceIds=[instance_id])
                    exit("Instance {0} rebooting.".format(instance_id))
            if action == 'terminate':
                if instance_state in ('rebooting', 'stopping',
                                      'terminated', 'shutting-down'):
                    exit("Cannot terminate instance {0} as state is {1}.".format(instance_id,
                                                                                 instance_state))
                elif instance_state in ('rebooting',
                                        'stopping', 'terminated',
                                        'shutting-down'):
                    exit("Instance {0} is already {1}.".format(instance_id, instance_state))
                else:
                    ec2_client.terminate_instances(InstanceIds=[instance_id])
                    exit("Instance {0} terminating.".format(instance_id))
    except AttributeError:
        exit("Cannot find instance: {0}".format(instance_id))


def ami_info(aws_config=None, ami_id=None):
    """
    @type aws_config: Config
    @type ami_id: unicode
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    try:
        output_ami_info(ami=ec2_client.describe_images(ImageIds=[ami_id]).get('Images')[0])
    except ClientError:
        exit('Unable to find ami: {0}'.format(ami_id))


def ami_list(aws_config):
    """
    @type aws_config: Config
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    output_ami_list(amis=ec2_client.describe_images(Owners=['self']).get('Images'))


def ec2_get_instance_vols(aws_config=None, instance_id=None):
    """
    @type aws_config: Config
    @type instance_id: unicode
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    reservations = ec2_client.describe_instances(InstanceIds=[instance_id])
    reservation = reservations.get('Reservations')[0]
    instance = reservation.get('Instances')[0]
    vols = list()
    for bdm in instance.get('BlockDeviceMappings'):
        vols.append(bdm)
    return vols



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
usage: acli [--version] [--help] [--install-completion]
       acli [--region=<region>] [--access_key_id=<access_key_id>] [--secret_access_key=<secret_access_key>]
            <command> [<args>...]
       acli account [options]
       acli ec2 (list | summary) [options]
       acli ec2 (start | stop | reboot | terminate | info | cpu | vols | net) <instance_id> [options]
       acli lc list [options]
       acli asg list [options]
       acli asg (info | cpu | mem | net | delete) <asg_name> [options]
       acli ami (list | info <ami_id>)
       acli eip (list | info <eip>)
       acli elb (list | info <elb_name>)
       acli route53 (list | info <zone_id>)
       acli vpc (list | info <vpc_id>)
       acli secgroup (list | info)
       acli s3 list [<item>]
       acli s3 info <item>

options:
   -h, --help      help

The most common commands are:
   account      Get account info
   ec2          Manage ec2 instances
   eip          Manage elastic IPs
   elb          Manage elb instances
   ami          Manage amis
   asg          Manage auto-scaling groups
   lc           Manage launch configurations
   eip          Manage elastic ips
   secgroup     Manage security groups
   route53      Manage route 53 configuration
   vpc          Manage VPCs
   secgroup     Manage Security Groups
   s3           Manage S3 storage
See 'acli help <command>'
"""

from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt
from acli.services import (ec2, elb, account, cloudwatch, vpc,
                           eip, asg, route53, secgroup, s3)
from acli.config import Config
from acli.utils import install_completion


def real_main():
    args = docopt(__doc__,
                  version='0.1.11',
                  options_first=True)
    aws_config = Config(args)
    argv = [args['<command>']] + args['<args>']
    if args['--install-completion']:
        install_completion()
    if args['<command>'] == 'account':
        from acli.commands import account as command_account
        # acc_res = docopt(command_account.__doc__, argv=argv)
        account.account_info(aws_config)
    if args['<command>'] == 'ec2':
        from acli.commands import ec2 as command_ec2
        ec2_res = docopt(command_ec2.__doc__, argv=argv)
        if ec2_res.get('list'):
            ec2.ec2_list(aws_config)
        elif ec2_res.get('info'):
            ec2.ec2_info(aws_config, instance_id=ec2_res.get('<instance_id>'))
        elif ec2_res.get('stop'):
            ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="stop")
        elif ec2_res.get('reboot'):
            ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="reboot")
        elif ec2_res.get('start'):
            ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="start")
        elif ec2_res.get('terminate'):
            ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="terminate")
        elif ec2_res.get('cpu'):
            cloudwatch.ec2_cpu(aws_config=aws_config, instance_id=ec2_res.get('<instance_id>'))
        elif ec2_res.get('net'):
            cloudwatch.ec2_net(aws_config=aws_config,
                               instance_id=ec2_res.get('<instance_id>'),
                               start=ec2_res.get('--start', None),
                               period=ec2_res.get('--end', None),
                               intervals=ec2_res.get('intervals', None)
                               )
        elif ec2_res.get('vols'):
            cloudwatch.ec2_vol(aws_config=aws_config,
                               instance_id=ec2_res.get('<instance_id>'),
                               start=ec2_res.get('--start', None),
                               period=ec2_res.get('--end', None),
                               intervals=ec2_res.get('intervals', None)
                               )
        elif ec2_res.get('summary'):
            ec2.ec2_summary(aws_config=aws_config)
    if args['<command>'] == 'elb':
        from acli.commands import elb as command_elb
        elb_res = docopt(command_elb.__doc__, argv=argv)
        if elb_res.get('list'):
            elb.elb_list(aws_config)
        elif elb_res.get('info'):
            elb.elb_info(aws_config, elb_name=elb_res.get('<elb_name>'))
    if args['<command>'] == 'lc':
        from acli.commands import lc as command_lc
        lc_res = docopt(command_lc.__doc__, argv=argv)
        if lc_res.get('list'):
            asg.lc_list(aws_config)
        elif lc_res.get('info'):
            asg.lc_info(aws_config, lc_name=lc_res.get('<lc_name>'))
    if args['<command>'] == 'asg':
        from acli.commands import asg as command_asg
        asg_res = docopt(command_asg.__doc__, argv=argv)
        if asg_res.get('list'):
            asg.asg_list(aws_config)
        elif asg_res.get('info'):
            asg.asg_info(aws_config, asg_name=asg_res.get('<asg_name>'))
        elif asg_res.get('cpu'):
            cloudwatch.asg_cpu(aws_config=aws_config,
                               asg_name=asg_res.get('<asg_name>'),
                               output_type=asg_res.get('--output', None),
                               start=asg_res.get('--start'),
                               period=asg_res.get('--end'),
                               intervals=asg_res.get('intervals')
                               )
        elif asg_res.get('delete'):
            asg.asg_delete(aws_config, asg_name=asg_res.get('<asg_name>'))
    if args['<command>'] == 'ami':
        from acli.commands import ami as command_ami
        ami_res = docopt(command_ami.__doc__, argv=argv)
        if ami_res.get('list'):
            ec2.ami_list(aws_config)
        elif ami_res.get('info'):
            ec2.ami_info(aws_config, ami_id=ami_res.get('<ami_id>'))
    if args['<command>'] == 'route53':
        from acli.commands import route53 as command_route53
        route53_res = docopt(command_route53.__doc__, argv=argv)
        if route53_res.get('list'):
            route53.route53_list(aws_config)
        elif route53_res.get('info'):
            route53.route53_info(aws_config, zone_id=route53_res.get('<zone_id>'))
    if args['<command>'] == 'vpc':
        from acli.commands import vpc as command_vpc
        vpc_res = docopt(command_vpc.__doc__, argv=argv)
        if vpc_res.get('list'):
            vpc.vpc_list(aws_config)
        elif vpc_res.get('info'):
            vpc.vpc_info(aws_config, vpc_id=vpc_res.get('<vpc_id>'))
    if args['<command>'] == 'secgroup':
        from acli.commands import secgroup as command_secgroup
        secgroup_res = docopt(command_secgroup.__doc__, argv=argv)
        if secgroup_res.get('list'):
            secgroup.secgroup_list(aws_config)
        elif secgroup_res.get('info'):
            secgroup.secgroup_info(aws_config, secgroup_id=secgroup_res.get('<secgroup_id>'))
    if args['<command>'] == 'eip':
        from acli.commands import eip as command_eip
        eip_res = docopt(command_eip.__doc__, argv=argv)
        if eip_res.get('list'):
            eip.eip_list(aws_config)
        elif eip_res.get('info'):
            eip.eip_info(aws_config, eip=eip_res.get('<eip>'))
    if args['<command>'] == 's3':
        from acli.commands import s3 as command_s3
        s3_res = docopt(command_s3.__doc__, argv=argv)
        if s3_res.get('list'):
            path = s3_res.get('<item>')
            if path and not '/' == path[-1:]:
                path += '/'
            s3.s3_list(aws_config, item=path)
        elif s3_res.get('info'):
            s3.s3_info(aws_config, item=s3_res.get('<item>'))
    elif args['<command>'] in ['help', None] and args['<args>']:
        if args['<args>'][0] == 'ec2':
            from acli.commands import ec2 as command_ec2
            print(docopt(command_ec2.__doc__, argv=argv))
    elif args['<command>'] in ['help', None] and not args['<args>']:
        print("usage: acli help <command>")
    else:
        exit("%r is not an acli command. See 'acli --help'." % args['<command>'])



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.route53 import (output_route53_list, output_route53_info)
import botocore.exceptions
from acli.connections import get_client


def route53_list(aws_config=None):
    """
    @type aws_config: Config
    """
    route53_client = get_client(client_type='route53', config=aws_config)
    zones = route53_client.list_hosted_zones()
    if zones.get('HostedZones', None):
        output_route53_list(zones=route53_client.list_hosted_zones())
    else:
        exit("No hosted zones found.")


def route53_info(aws_config=None, zone_id=None):
    """
    @type aws_config: Config
    @type zone_id: unicode
    """
    route53_client = get_client(client_type='route53', config=aws_config)
    try:
        hosted_zone = route53_client.get_hosted_zone(Id=zone_id)
        record_sets = route53_client.list_resource_record_sets(HostedZoneId=zone_id)
        if hosted_zone['HostedZone']['Id']:
            output_route53_info(zone=hosted_zone,
                                record_sets=record_sets)
    except botocore.exceptions.ClientError:
        exit("Cannot request hosted zone: {0}".format(zone_id))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import (output_ascii_table, output_ascii_table_list, dash_if_none)
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def get_tag(name=None, tags=None):
    if tags:
        for tag in tags:
            if tag.get('Key') == name:
                return tag.get('Value')


def output_vpc_list(vpcs=None):
    """
    @type vpcs: dict
    """
    td = list()
    table_header = [Color('{autoblue}vpc id{/autoblue}'), Color('{autoblue}name{/autoblue}'),
                    Color('{autoblue}CIDR block{/autoblue}'), Color('{autoblue}tenancy{/autoblue}'),
                    Color('{autoblue}state{/autoblue}'), Color('{autoblue}DHCP options{/autoblue}'),
                    Color('{autoblue}default vpc{/autoblue}')]
    for vpc in vpcs.get('Vpcs'):
        vpcid = vpc.get('VpcId')
        cidr_block = vpc.get('CidrBlock')
        tenancy = vpc.get('InstanceTenancy')
        state = vpc.get('State')
        dhcpoptions = vpc.get('DhcpOptionsId')
        default = str(vpc.get('IsDefault'))
        td.append([vpcid,
                   dash_if_none(get_tag(name='Name', tags=vpc.get('Tags', None))),
                   dash_if_none(cidr_block),
                   dash_if_none(tenancy),
                   dash_if_none(state),
                   dash_if_none(dhcpoptions),
                   default])
    output_ascii_table_list(table_title=Color('{autowhite}VPCs{/autowhite}'),
                            table_data=td,
                            table_header=table_header,
                            inner_heading_row_border=True)
    exit(0)


def output_vpc_info(vpc=None, subnets=None):
    """
    @type vpc: ec2.Vpc
    @type subnets: list
    """
    if vpc:
        td = list()
        td.append([Color('{autoblue}vpc id{/autoblue}'), vpc.get('VpcId')])
        td.append([Color('{autoblue}CIDR block{/autoblue}'), vpc.get('CidrBlock')])
        td.append([Color('{autoblue}default{/autoblue}'), str(vpc.get('IsDefault'))])
        td.append([Color('{autoblue}tenancy{/autoblue}'), vpc.get('InstanceTenancy')])
        td.append([Color('{autoblue}state{/autoblue}'), dash_if_none(vpc.get('State'))])
        td.append([Color('{autoblue}tags{/autoblue}'), " "])
        if vpc.get('Tags'):
            for vpc_tag in vpc.get('Tags'):
                td.append([Color('{autoblue}'+ "{0}".format(vpc_tag.get('Key'))+'{/autoblue}'),
                           " {0}".format(vpc_tag.get('Value'))])
        if subnets:
            td.append(["{0}".format('-' * 30), "{0}".format('-' * 30)])
            td.append([Color('{autowhite}SUBNETS{/autowhite}'), " "])
            for subnet in subnets.get('Subnets'):
                td.append(["{0}".format('-' * 30),
                           "{0}".format('-' * 30)])
                td.append([Color('{autoblue}subnet id{/autoblue}'),
                           subnet.get('SubnetId')])
                td.append([Color('{autoblue}az{/autoblue}'),
                           subnet.get('AvailabilityZone')])
                td.append([Color('{autoblue}state{/autoblue}'),
                           subnet.get('State')])
                td.append([Color('{autoblue}available IPs{/autoblue}'),
                           str(subnet.get('AvailableIpAddressCount'))])
                td.append([Color('{autoblue}CIDR block{/autoblue}'),
                           subnet.get('CidrBlock')])
                td.append([Color('{autoblue}default for az{/autoblue}'),
                           str(subnet.get('DefaultForAz'))])
                td.append([Color('{autoblue}map public ip on launch{/autoblue}'),
                           str(subnet.get('MapPublicIpOnLaunch'))])
                if subnet.get('Tags'):
                    td.append([Color('{autoblue}tags{/autoblue}'), "-"])
                    for tag in subnet.get('Tags'):
                        tag_key, tag_value = dash_if_none(tag.get('Key', None)), dash_if_none(tag.get('Value', None))
                        td.append([Color('{autoblue}'+" {}".format(tag_key)+'{/autoblue}'), "{}".format(tag_value)])
        output_ascii_table(table_title=Color('{autowhite}vpc info{/autowhite}'),
                           table_data=td)
    else:
        exit('VPC does not exist.')
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from boto3.session import Session
from contextlib import contextmanager
from botocore.exceptions import NoCredentialsError, NoRegionError


def exit_with_credentials_message():
    exit('Credentials not found. See here for more information:\n'
         'http://boto3.readthedocs.org/en/latest/guide/configuration.html#configuration-files')


@contextmanager
def checked_autoscaling_client(autoscaling_client):
    try:
        assert autoscaling_client.describe_launch_configurations()
        yield autoscaling_client
    except NoCredentialsError:
        exit_with_credentials_message()
    except Exception as e:
        exit('Unhandled exception: {0}'.format(e))


@contextmanager
def checked_iam_client(iam_client):
    try:
        assert iam_client.list_users()
        yield iam_client
    except NoCredentialsError:
        exit_with_credentials_message()
    except Exception as e:
        exit('Unhandled exception: {0}'.format(e))


@contextmanager
def checked_ec2_client(ec2_client):
    try:
        assert ec2_client.describe_instances(MaxResults=5)
        yield ec2_client
    except NoCredentialsError:
        exit_with_credentials_message()
    except Exception as e:
        exit('Unhandled exception: {0}'.format(e))


@contextmanager
def checked_elb_client(elb_client):
    try:
        assert elb_client.describe_load_balancers()
        yield elb_client
    except NoCredentialsError:
        exit_with_credentials_message()
    except Exception as e:
        exit('Unhandled exception: {0}'.format(e))


@contextmanager
def checked_cloudwatch_client(cloudwatch_client):
    try:
        assert cloudwatch_client.describe_alarms(MaxRecords=1)
        yield cloudwatch_client
    except NoCredentialsError:
        exit_with_credentials_message()
    except Exception as e:
        exit('Unhandled exception: {0}'.format(e))


@contextmanager
def checked_route53_client(route53_client):
    try:
        assert route53_client.list_hosted_zones(MaxItems='1')
        yield route53_client
    except NoCredentialsError:
        exit_with_credentials_message()
    except Exception as e:
        exit('Unhandled exception: {0}'.format(e))


@contextmanager
def checked_s3_client(s3_client):
    try:
        assert s3_client.list_buckets()
        yield s3_client
    except NoCredentialsError:
        exit_with_credentials_message()
    except Exception as e:
        exit('Unhandled exception: {0}'.format(e))


def get_boto3_session(aws_config):
    """
    @type aws_config: Config
    """
    if all((aws_config.region, aws_config.access_key_id, aws_config.secret_access_key)):
        return Session(region_name=aws_config.region,
                       aws_access_key_id=aws_config.access_key_id,
                       aws_secret_access_key=aws_config.secret_access_key)
    elif aws_config.region:
        return Session(region_name=aws_config.region)
    else:
        return Session()


def get_client(client_type=None, config=None):
    """
    @type client_type: basestring
    @type config: Config
    """
    session = get_boto3_session(aws_config=config)
    try:
        if client_type == 'ec2':
            with checked_ec2_client(session.client('ec2')) as ec2_client:
                return ec2_client
        elif client_type == 'elb':
            with checked_elb_client(session.client('elb')) as elb_client:
                return elb_client
        elif client_type == 'iam':
            with checked_iam_client(session.client('iam')) as iam_client:
                return iam_client
        elif client_type == 'autoscaling':
            with checked_autoscaling_client(session.client('autoscaling')) as autoscaling_client:
                return autoscaling_client
        elif client_type == 'cloudwatch':
            with checked_cloudwatch_client(session.client('cloudwatch')) as cloudwatch_client:
                return cloudwatch_client
        elif client_type == 'route53':
            with checked_route53_client(session.client('route53')) as route53_client:
                return route53_client
        elif client_type == 's3':
            with checked_s3_client(session.client('s3')) as s3_client:
                return s3_client
    except NoRegionError:
        exit('Cannot perform this task without specifying an AWS region.\n'
             'Please check your boto/aws settings or specify using \'acli --region=<region>\'.')
    except Exception as e:
        exit('Unhandled exception: {0}'.format(e))
